//      by [GDS|Entropy]
#include "SnowIce.inc"

global_settings {
 max_trace_level 256
}

camera {
 location <-5, 6,-17>
 up y*image_height right x*image_width
 look_at < 0, 1.5, 0>
 angle 45
} 

light_source {
  0*x                  
  color rgb <1,1,1>    
  translate <-20, 40, -20>
}

#declare cyl =
cylinder {
 <-2,0,0>,<2,0,0>, 1
 translate <0,0,0>
 scale 2 
} 

#declare sph = 
sphere {
 <0,0,0>, 1
 translate <0,0,0>
 scale 1
}

#declare sphcyl = 
merge {
 object {
  sph
  translate <0,-1,0>
 }
 object {
  cyl
 }
}

#declare filterVal = 0.85;

#declare Ice2 = 
texture {
 pigment {
  bozo 
  octaves 1 
  frequency 4 
  triangle_wave
  color_map { 
   [0.00 rgbt <0.97,0.97,0.98,filterVal+0.1>]
   [0.03 rgbt <0.91,0.97,0.97,filterVal+0.1>]
   [0.40 rgbt <0.91,0.91,0.93,filterVal+0.1>]
   [0.41 rgbt <0.91,0.94,0.94,filterVal+0.1>]
   [0.42 rgbt <0.91,0.91,0.97,filterVal+0.1>]
   [1.00 rgbt <0.91,0.91,0.97,filterVal+0.1>]
  }
 }
 finish { 
  ambient 0 
  diffuse 1 
  brilliance 2 
  specular 0.5 
  roughness 0.001 
  reflection {
   0.05, 0.25
  }          
  conserve_energy
 }
}

#declare iceInterior =
  interior {
   fade_distance 2
   fade_power 2
   ior 1.309 // ior of ice
   caustics 1.0 
  } 

//------[Macro Definition]------//
//#macro SnowIce(ctr,RsA,vRangeMin,vRangeMax,vGenType,vGenMethod,testVector,testObject,radVal,strVal,useTexture,useInterior,icicleSegs,icicleProbability,icicleTexture,snowTexture,iceInterior,upcastCount)
//------[Macro Definition]------// 

//----------[   Macro Usage: Example Call       ]----------//
#declare targetObj = cyl;

#declare st0 = 
SnowIce(
    5000,           //  Base number of particles; should match the size of the random vector array.
    seed(574647),   //  Random seed.
    -4,             //  Random vector minimum range; based on object size.
    4,              //  Random vector maximum range; based on object size.
    0,              //  Random vector generator type. (0,1) 
    0,              //  Random vector generator method (0-5)
    <0,12,0>,       //  Base vector used by Trace()
    targetObj,      //  The object to encrust.
    1,              //  Base Blob radius value.
    1,              //  Base Blob strength value.
    false,          //  Use Texture?
    false,          //  Use Interior?
    true,           //  Use Snow?
    12,             //  Number of Icicle segments.
    5,              //  Base probability of Icicle occurance. 0 = off
    Ice2,           //  Icicle Texture.
    Ice2,           //  Snow Texture.
    iceInterior,    //  The interior to apply.
    30              //  Base upcast particle count.
    );

//  NOTE:
//  If you choose to apply texture and interior to snow/ice individually, 
//  expect *very* long render times! Otherwise, apply to entire object.
//  You *can* call the macro twice, once on the target object with snow
//  off and icicles on, then once on the union of the target object and
//  the output of the macro with icicles off. THis way you can use 
//  different textures for the ice and the snow.

//----------[   Macro Usage: End Example Call   ]----------//

#declare snowyObject =
union {
 object { 
  st0   
  texture {
   Ice2
  }
  interior {
   iceInterior
  }  
 } 
 object {
  targetObj
  texture { 
   pigment {
    color rgb <145/255,102/255,32/255>
   }  
  }
  finish { 
   ambient 0 
   diffuse 1 
   specular 0.5 
   roughness 0.01 
   reflection 0.03
  }
  normal { 
   wrinkles 0.2 
   scale 0.5 
  }       
 }                
 translate y * 3
 rotate y * 45
}

object {
 snowyObject
}

plane {
 y, 0
 texture {
  pigment {
   color rgb 1   
  } 
  finish {
   diffuse 0.65
   ambient 0
  }
 } 
 translate y * -5
}
   